% DAILYMEANS calculate daily means of high frequency data
%
% [HD,DATESD]=DAILYMEANS(H,DATES)
% [HD,DATESD]=DAILYMEANS(H,DATES,OFFSET,USENANMEAN,NANVAL)
%
% Returns dates at the start of the day
% Use offset= 0.5 to return dates at noon
% if USENANMEAN=0 then it will return nan for any day containing a nan
% 
% NANVAL can be specified (eg it might be 1e17)
% 
% THIS VERSION ONLY TAKES VECTOR H, unlike the one in WEIGHING
% (they need merging)
% 
% Joanne Williams, joll@noc.ac.uk, 2013



function [hdaily,datesdaily]=dailymeans(heightsin,datesin,offset,usenanmean)



%Ensure input(s) are column vectors, if not output is in the same format
if isrow(heightsin)
    returnrowheights=1;
else
    returnrowheights=0;
end

if isrow(datesin)
    returnrowdates=1;
else
    returnrowdates=0;
end

datesin=datesin(:);
heightsin=heightsin(:);


%returns dates at the start of the day
%eg use offset= 0.5 to return dates at midday
if nargin<3, offset=0; end
if nargin<4, usenanmean=0; end


[~,ia,ic]=unique(floor(datesin),'first');
datesdaily=datesin(ia)+offset;

Ndm=length(datesdaily);
hdaily=zeros(Ndm,1);
if usenanmean
    for i=1:Ndm
  %      hday=h(ic==i);
        hdaily(i)=nanmean(heightsin(ic==i));
    end
else
    for i=1:Ndm
        hdaily(i)=mean(heightsin(ic==i));
    end
end


if returnrowdates
    datesdaily=datesdaily(:)';
end
if returnrowheights
    hdaily=hdaily(:)';
end