function sitemetadata=readIOCmeta(datadir,sitecode)

%READIOCMETA Read metadata from IOC site for a given site
% 
% Output and and save locally in datadir. 
%
% This function is required for TIDEGAUGEQC.
% 
% Joanne Williams, joll@noc.ac.uk, March 2019


try 
   load([datadir,sitecode,'_metadata.mat'],'sitemetadata')
   
catch
    
sitemetadata=webread('http://www.ioc-sealevelmonitoring.org/service.php?',...
        'query','station','format','ascii',...
        'code',sitecode);
    
    if ~exist(datadir,'dir')
        mkdir(datadir)
    end
    save([datadir,sitecode,'_metadata.mat'],'sitemetadata')
    
end
%Then eg 
%latitude=sitemetadata.Lat;
%
% %Example fields:
%                    Code: 'calq'
%                 Location: 'Calliaqua Coast Guard Base'
%                  country: 'SVG'
%                     type: 'SL'
%                   DCP_ID: '53500294'
%                      WMO: 'SOVG10'
%                   XMtInt: 5
%                      Lat: 13.1299
%                      Lon: -61.1955
%                      Har: []
%                  GlossID: []
%                 OT12Code: []
%                  Example: []
%                UTCOffset: 0
%               OperatorID: []
%                   status: 'Operational'
%       status_description: 'Data logger was damaged, OTT is sending a replacement (Christa von Hillebrandt…'
%            localoperator: 'National Emergency Management Office ( Saint Vincent & Grenadines )'
%     performanceoperator1: 'Caribbean Tsunami Warning Program ( USA )'
%     performanceoperator2: []
%     performanceoperator3: []
%             date_created: '2013-12-12 12:29:19           '
%                      url: []
%               statsensor: []
%                     code: []
%               updatedata: []
%            timestampdata: []
%          timestampupdate: []
%             transmittype: 'GTS message'
%              countryname: 'Saint Vincent & Grenadines'
%                  example: 'SOVG10 KWAL 261453…'
%                  totmess: 0
%             expectedmess: 288
%                  sensors: [3×1 struct]
