function [singlechanneldates,singlechannelheights,singlechannelflags,singlechannelorigin]=...
    stitchbestchannel(chanvec,chanrank,dateS,dateE,datescell,heightscell,...
    flagbadcell,chanflagprefcell,datumcor_afterqc)

%STITCHBESTCHANNEL Attempt to stitch together the best data possible from multiple channels.
%
% For many applications it may be necessary to return a single unified channel.
%Attempt to stitch together the best data possible from multiple channels.
%
%Datum Correction is *ESSENTIAL*. If datum correction is not known
%exactly, it should be recalculated as the median of only the consistent
%data in each channel (hopefully then omitting poor data).
%
% Joanne Williams 2019

%
%It may be that the preferred channel information is specified in advance
%by the operator, or we may have to derive it.
%
%Where there are more than one channel that passes the earlier tests and
%has chanflagprefcell{ichan}==1,
%then we cannot necessarily state which channel is "best".
%(Recall that we only used this earlier to select the best where there were
%no similar channels)
%
% So we make an arbitrary decision. This is that the channel with the most
% data overall is the default selection. Rank all channels according to this
% criteria. If this channel is missing
% chanflagprefcell{ichan}==0 for a given date, or there is flagged data,
% then instead select the next best option.


%Ideally we want a high frequency single channel.

%We must also return which channel this information came from.

plotfigs=0;

gaplengthtoswitch=12/24; %only consisder switching if there's at least a 12 hour gap
avoidoverlap=2/(24*60); %2 min

Nchan=length(chanvec);

%% Start with the highest ranking channel.
ichan1=chanvec(chanvec==chanrank(1));

singlechanneldates=datescell{ichan1}(flagbadcell{ichan1}==0&chanflagprefcell{ichan1}==1);
singlechannelheights=heightscell{ichan1}(flagbadcell{ichan1}==0&chanflagprefcell{ichan1}==1)-datumcor_afterqc{ichan1};
singlechannelflags=flagbadcell{ichan1}(flagbadcell{ichan1}==0&chanflagprefcell{ichan1}==1);
singlechannelorigin=repmat(chanrank(1),size(singlechanneldates));

if plotfigs
    figure,
    plotwithgaps(singlechanneldates,singlechannelheights)
    hold on
end

if Nchan==1
    return
end


%% Now include the next highest ranking channel, where there is a gap.
%Ensure it is stitched in the right dates, and datum corrected
ichan2=chanvec(chanvec==chanrank(2));

dates=datescell{ichan2};
heights=heightscell{ichan2}-datumcor_afterqc{ichan2};
flags=flagbadcell{ichan2};
origin=repmat(chanrank(2),size(dates));
chanflagpref=chanflagprefcell{ichan2};

%if there is a date for which there is a gap in singlechanneldates but
%there is data in dates then fill it in. Only switch channels if the gap is
%at least 1 day.

dS=singlechanneldates(1);
dE=singlechanneldates(end);
dgapS=singlechanneldates(find(diff(singlechanneldates)>gaplengthtoswitch));
dgapE=singlechanneldates(find(diff(singlechanneldates)>gaplengthtoswitch)+1);

Ngap=length(dgapS);


if Ngap>0
    %list of potential dates to add:
    idatesextra=1:length(dates);
    %avoidoverlap is to ensure we don't add a single date point that's just a
    %minute outside the range
    idates=dS-avoidoverlap<=dates&dates<=dgapS(1)+avoidoverlap; %We won't want to add dates where they're in the singlechannel already
    idatesextra(idates)=nan;
    
    for igap=1:Ngap-1
        idates=dgapE(igap)-avoidoverlap<=dates&dates<=dgapS(igap+1)+avoidoverlap;
        idatesextra(idates)=nan;
    end
    idates=dgapE(Ngap)-avoidoverlap<=dates&dates<=dE+avoidoverlap;
    idatesextra(idates)=nan;
    idatesextra(isnan(idatesextra))=[];
    
    %Remove those dates that don't have a good channel with them
    idatesextra(chanflagpref(idatesextra)~=1)=[];
    
    
    %Remove those dates that are flagged
    idatesextra(flags(idatesextra)==1)=[];
    
    
    [newdates,isort]=sort([singlechanneldates;dates(idatesextra)],'ascend');
    newheights=[singlechannelheights;heights(idatesextra)]; %UNSORTED!
    newheights=newheights(isort);
    
    newflags=[singlechannelflags;flags(idatesextra)]; %UNSORTED!
    newflags=newflags(isort);
    
    neworigin=[singlechannelorigin;origin(idatesextra)]; %UNSORTED!
    neworigin=neworigin(isort);
    
    
    
    singlechanneldates=newdates;
    singlechannelheights=newheights;
    singlechannelflags=newflags;
    singlechannelorigin=neworigin;
    
    
    if plotfigs
        
        plot(dates(idatesextra),heights(idatesextra),'.')
        
    end
    
end

if Nchan==2
    return
end





%% And again for the next channel
%This is identical, so it could be looped.
%Ensure it is stitched in the right dates, and datum corrected
ichan3=chanvec(chanvec==chanrank(3));

dates=datescell{ichan3};
heights=heightscell{ichan3}-datumcor_afterqc{ichan3};
flags=flagbadcell{ichan3};
origin=repmat(ichan3,size(dates));
chanflagpref=chanflagprefcell{ichan3};

%if there is a date for which there is a gap in singlechanneldates but
%there is data in dates then fill it in. Only switch channels if the gap is
%at least 1 day (gaplengthtoswitch).


dS=singlechanneldates(1);
dE=singlechanneldates(end);
dgapS=singlechanneldates(find(diff(singlechanneldates)>gaplengthtoswitch));
dgapE=singlechanneldates(find(diff(singlechanneldates)>gaplengthtoswitch)+1);

Ngap=length(dgapS);

if Ngap>0
    idatesextra=1:length(dates);
    idates=dS-avoidoverlap<=dates&dates<=dgapS(1)+avoidoverlap; %We won't want to add dates where they're in the singlechannel already
    idatesextra(idates)=nan;
    
    for igap=1:Ngap-1
        idates=dgapE(igap)-avoidoverlap<=dates&dates<=dgapS(igap+1)+avoidoverlap;
        idatesextra(idates)=nan;
    end
    idates=dgapE(Ngap)-avoidoverlap<=dates&dates<=dE+avoidoverlap;
    idatesextra(idates)=nan;
    idatesextra(isnan(idatesextra))=[];
    
    %Remove those dates that don't have a good channel with them
    idatesextra(chanflagpref(idatesextra)~=1)=[];
    
    
    %Remove those dates that are flagged
    idatesextra(flags(idatesextra)==1)=[];
    
    [newdates,isort]=sort([singlechanneldates;dates(idatesextra)],'ascend');
    newheights=[singlechannelheights;heights(idatesextra)]; %UNSORTED!
    newheights=newheights(isort);
    
    newflags=[singlechannelflags;flags(idatesextra)]; %UNSORTED!
    newflags=newflags(isort);
    
    neworigin=[singlechannelorigin;origin(idatesextra)]; %UNSORTED!
    neworigin=neworigin(isort);
    
    
    
    singlechanneldates=newdates;
    singlechannelheights=newheights;
    singlechannelflags=newflags;
    singlechannelorigin=neworigin;
    
    if plotfigs
        
        plot(dates(idatesextra),heights(idatesextra),'.')
        
    end
end
if Nchan==3
    return
end




