% STUCKFLAGS  Stability test: flagging values when there is
% no change in the magnitude of sea level after
% a number of time steps. The number of data
% values or time steps to begin to flag should depend
%  on the time interval and data precision. A typical
% value, for example, is 3 for 5 minute data.
%
% However this test is currently fixed at flagging for 5 values. 
%
%
% If there is no change across 5 values,
% Don't flag the first of the stuck ones (unless flagged due to previous
% stuck)
% flag the 2nd of the 5
% also the 3rd-4th-5th of the 5 
%
% Tolerance is set to STUCKTOL=0.0001
%
% Dates are ignored - if there is the same value before and after a big
% data gap it's just as suspect as without the gap. 
%
% Repeated NaN are ignored
%
% st lucia pr1 starts off jumping between fixed values. 
% Not stuck, but not right! For this use stuck oscillating test
%
% STUCKFLAG = 1 for GOOD data, 4 for BAD data.
%
% This function is required for TIDEGAUGEQC.
% 
% Joanne Williams joll@noc.ac.uk Nov 2018

function  stuckflag=stuck_detect_test(heights,timedelta,precisionest)

if nargin==3
    error('Stuck test not implemented for more than two arguments')
    %A possible upgrade is to adjust the stuck test according to the data
    %precision
elseif nargin==2
    if abs(timedelta-1)>0.5 %timedelta not approximately 1 minute?
        sprintf('Estimated sample interval %f minutes.',timedelta) 
         warning('Stuck test assuming 1 minute data and flagging 5 in a row.')
    end
else
    warning('Stuck test assuming 1 minute data and flagging 5 in a row.')
end

  
    %     %stuck test is currently reporting 5 in a row. This is too sensitive
    %     %for the radar gauge at 1 minute data and fairly coarse precision
    %     %- you can get 5 in a row randomly.
    %     %Adjust for longer time
    %On the other hand, if this is too long the spike detection can also
    %miss them!
   

stucktol=0.0001;

%if a point is the same as the following one, and the one after that, and
%the one after that, then flag the original and all the following until
%there is a change. 
%we have to set a tolerance - say same within 0.00001m
%This has to be quite small as the data might be only recorded to 1mm


if isrow(heights)
    returnrow=1; 
else 
    returnrow=0;
end

%Ensure column
heights=heights(:);

N=length(heights);

isstuck=false(N,1);


heightscompare=[heights(1:N-4)-heights(2:N-3),...
    heights(1:N-4)-heights(3:N-2),...
    heights(1:N-4)-heights(4:N-1),...
    heights(1:N-4)-heights(5:N)];

next5stuck=all(abs(heightscompare)<stucktol,2);

%If there is no change across 5 values,
%Don't flag the first of the stuck ones (unless flagged due to previous
%stuck)
%flag the 2nd of the 5
%also the 3rd-4th-5th of the 5 
isstuck(2:N-3)=next5stuck(1:N-4);
isstuck(3:N-2)=isstuck(3:N-2)|next5stuck(1:N-4);
isstuck(4:N-1)=isstuck(4:N-1)|next5stuck(1:N-4);
isstuck(5:N)=isstuck(5:N)|next5stuck(1:N-4);

% 

if returnrow
    isstuck=isstuck(:)';
end

stuckflag=ones(size(isstuck)); %GOOD
stuckflag(isstuck)=4; %BAD
