% STUCKOSCFLAGS Oscillating test: flagging values when there is
% sea level oscillating between a few fixed values for 
% a number of time steps. 
% 
% STUCKOSCFLAGS=STUCK_OSC_DETECT_TEST(HEIGHTS,PRECISIONEST)
%
% Default Precision = 0.0001
% Default tolerance is set to STUCKTOL=PRECISIONEST/10;
%
% Dates are ignored - if there is the same value before and after a big
% data gap it's just as suspect as without the gap. 
% Last M points in the series are ignored
%
% Default M=60;
% Default number repeats to flag is 20  or more in 60 data points.
%
% Repeated NaN are ignored
%
% Example:
% st lucia pr1 starts off jumping between fixed values. 
% Not stuck, but not right! For this use stuck oscillating test
%
% FLAGS: 1 for GOOD data, 4 for BAD data.
%
% This function is required for TIDEGAUGEQC.
%
% Joanne Williams Nov 2018

function  stuckoscflag=stuck_osc_detect_test(heights,precisionest)

if nargin<2
    precisionest=0.0001; %ie 0.1mm, estimated precision (NOT ACCURACY!) of data.
end
stucktol=precisionest/10; %quick handling tolerance in case of floating point
M=60;
suspectrepno=20; %how many times is a value be repeated in the next M values
%to be suspicious


if isrow(heights)
    returnrow=1; 
else 
    returnrow=0;
end

%Ensure column
heights=heights(:);

N=length(heights);

isstuckosc=false(N,1);

heightscompare=zeros(N-M,M);
for i=1:M
    heightscompare(:,i)=heights(1:N-M)-heights(i+1:N-M+i);
end


%How many times does this value come up in the next M values?
repeatplaces=abs(heightscompare)<stucktol;
thisvalueinthenextM=sum(repeatplaces,2);

%If that's more than suspectrepno times, flag this value
thisvaluerepeats=(thisvalueinthenextM>=suspectrepno);

isstuckosc(1:N-M)=thisvaluerepeats;

idx_last_nonzero = M - sum( cumprod(fliplr(repeatplaces) == 0, 2), 2);
%Also flag the following  until the last occurrence of repeat
for i=1:M
    stillrepeating= (i<=idx_last_nonzero );
    isstuckosc(i+1:N-M+i)=isstuckosc(i+1:N-M+i)|(thisvaluerepeats&stillrepeating);
end



if returnrow
    isstuckosc=isstuckosc(:)';
end

%1 for GOOD data, 4 for BAD data.
stuckoscflag=ones(size(isstuckosc));
stuckoscflag(isstuckosc)=4; 